/** @file KFile.h
* Basic file handle interface.
*
*	K3dEngine 1.0 - OpenGL Game Engine
*	http://sourceforge.net/projects/kengine/
*
* 06.03.23 - First version
* 
*
*/

#pragma once

namespace K3d
{
	class CFile
	{	
			FILE*	m_pFile;	///< Pointer to file
		protected:
			///\brief Open modes for binary read
			///\param _rFileName Reference to filename string
			///\retval bool True if file exists and opened
			bool OpenForRead(const std::string& _rFileName)	
			{ 
				m_pFile = fopen(_rFileName.c_str(), "rb"); 
				return Good(); 
			}
			///\brief Open file for binary write
			///\param _rFileName Reference to filename string
			///\retval bool True if file exists and opened
			bool OpenForWriteTrunc(const std::string& _rFileName)	
			{ 
				m_pFile = fopen(_rFileName.c_str(), "wb"); 
				return Good(); 
			}
			///\brief Open file for append
			///\param _rFileName Reference to filename string
			///\retval bool True if file exists and opened
			bool OpenForWriteAppend(const std::string& _rFileName)
			{
				m_pFile = fopen(_rFileName.c_str(), "ab");
				return Good(); 
			}
			///\brief Open file for text read
			///\param _rFileName Reference to filename string
			///\retval bool True if file exists and opened
			bool OpenForReadText(const std::string& _rFileName)	
			{ 
				m_pFile = fopen(_rFileName.c_str(), "rt");
				return Good(); 
			}
			///\brief Open modes for text read
			///\param _rFileName Reference to filename string
			///\retval bool True if file exists and opened
			bool OpenForWriteTextTrunc(const std::string& _rFileName)	
			{ 
				m_pFile = fopen(_rFileName.c_str(), "wt"); 
				return Good(); 
			}
			///\brief Open modes for text append
			///\param _rFileName Reference to filename string
			///\retval bool True if file exists and opened
			bool OpenForWriteTextAppend(const std::string& _rFileName)
			{ 
				m_pFile = fopen(_rFileName.c_str(), "at"); 
				return Good(); 
			}
		
		public:
			CFile() : m_pFile(0)	{}
			~CFile()	
			{ 
				if (m_pFile) fclose(m_pFile); 
			}
			
			///\brief Get pointer to file
			FILE* GetFile()
			{
				return m_pFile;
			}
			///\brief Close file
			///\retval bool True if correctly closed
			bool Close()
			{ 
				assert(Good());
				return (fclose(m_pFile) == 0); 
			}
			///\brief Read first word from file
			///\retval bool True if correctly flushed
			bool Flush()
			{ 
				assert(Good());
				return (fflush(m_pFile) == 0);
			}
			///\brief Test if file is good
			///\retval bool True if file is good
			bool Good() const	
			{ 
				return (m_pFile != 0); 
			}
			///\brief Test if file is bad
			///\retval bool True if file is bad
			bool Bad() const	
			{
				return (m_pFile == 0); 
			}
			///\brief Test if file exists
			///\retval bool True if file exists
			bool Exists(const std::string& _rFileName)
			{ 
				FILE* pFile; 
				bool exists = ((pFile = fopen(_rFileName.c_str(), "rb")) != 0); 
				if (pFile) fclose(pFile); 
				return exists; 
			}
			///\brief File length    (only use before reading/writing)
			///\retval long File length
			long Length()
			{ 
				assert(Good()); 
				fseek(m_pFile, 0, SEEK_END); 
				long lLen = ftell(m_pFile); 
				fseek(m_pFile, 0, SEEK_SET); 
				return lLen; 
			}
	};
}	/// End namespace K3d
