/** @file KWriteFile.h
* File handle used only for binary-mode writing.
* Provides an interface for writing basic data-types and strings.
* When opening a file which already exists, new data written to this file will erase the old.
* Dynamic-binding not supported.
*
*	K3dEngine 1.0 - OpenGL Game Engine
*	http://sourceforge.net/projects/kengine/
*
* 06.03.23 - First version
* 
*
*/

#pragma once

namespace K3d
{
	class CWriterFile : public CFile
	{
	public:
		CWriterFile()	{}
		CWriterFile(const std::string& _rFileName)	
		{ 
			Open(_rFileName); 
		}
		CWriterFile(const std::string& _rFileName, bool _bAppend)	
		{ 
			Open(_rFileName, _bAppend); 
		}
		///\brief Open file for writing
		///\param _rFileName Reference to file
		bool Open(const std::string& _rFileName)	
		{ 
			return OpenForWriteTrunc(_rFileName); 
		}
		///\brief Open file for writing
		///\param _rFileName Reference to file
		///\param _bAppend Open for append write?
		bool Open(const std::string& _rFileName, bool _bAppend)	
		{ 
			if (_bAppend) 
			{
				return OpenForWriteAppend(_rFileName); 
			}
			else 
			{
				return OpenForWriteTrunc(_rFileName); 
			}
		}
		///\brief Open file for writing
		///\param _strBuf String buffer
		///\retval int Total number of writed items
		///\retval _strBuf File string
		int Write(const char* _srcBuf, size_t _tLen)	
		{ 
			assert(Good()); 
			return fwrite(reinterpret_cast<const void*>(_srcBuf), sizeof(char), _tLen, GetFile()); 
		}
		///\brief Write string
		///\retval int Total number of writed items
		int Write(const std::string& _str)			
		{ 
			long lLen = _str.length(); 
			int iVal = Write(lLen); 
			return (iVal+Write(_str.data(), lLen)); 
		}
		///\brief Write char
		///\param _cVal Char value 
		///\retval int Total number of writed items
		int Write(char _cVal)
		{ 
			return Write(reinterpret_cast<const char*>(&_cVal), sizeof(char)); 
		}
		///\brief Write unsigned char
		///\param _ucVal Unsigned Char value 
		///\retval int Total number of writed items
		int Write(unsigned char _ucVal)
		{ 
			return Write(reinterpret_cast<const char*>(&_ucVal), sizeof(unsigned char)); 
		}
		///\brief Write short
		///\param _sVal Short value 
		///\retval int Total number of writed items
		int Write(short _sVal)						
		{ 
			return Write(reinterpret_cast<const char*>(&_sVal), sizeof(short)); 
		}
		///\brief Write unsigned short
		///\param _usVal unsigned short value 
		///\retval int Total number of writed items
		int Write(unsigned short _usVal)				
		{ 
			return Write(reinterpret_cast<const char*>(&_usVal), sizeof(unsigned short)); 
		}
		///\brief Write long
		///\param _lVal long value 
		///\retval int Total number of writed items
		int Write(long _lVal)							
		{ 
			return Write(reinterpret_cast<const char*>(&_lVal), sizeof(long)); 
		}
		///\brief Write unsigned 
		///\param _ulVal unsigned long value 
		///\retval int Total number of writed items
		int Write(unsigned long _ulVal)				
		{ 
			return Write(reinterpret_cast<const char*>(&_ulVal), sizeof(unsigned long)); 
		}
		///\brief Write size_t 
		///\param _tVal size_t value 
		///\retval int Total number of writed items
		int Write(size_t _tVal)						
		{ 
			return Write(reinterpret_cast<const char*>(&_tVal), sizeof(size_t)); 
		}
		///\brief Write float
		///\param _fVal size_t value 
		///\retval int Total number of writed items
		int Write(float _fVal)						
		{ 
			return Write(reinterpret_cast<const char*>(&_fVal), sizeof(float)); 
		}
		///\brief Write double
		///\param _dVal double value 
		///\retval int Total number of writed items
		int Write(double _dVal)						
		{ 
			return Write(reinterpret_cast<const char*>(&_dVal), sizeof(double)); 
		}
	};
}	/// End namespace K3d
